@@ -18,11 +18,11 @@
  *
  * Created 17.6.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 22 May 02 14:10:38 tews@ithif51>
  *
  * the adt theories
  *
- * $Id: adt_theory.ml,v 1.12 2002/01/24 14:43:58 tews Exp $
+ * $Id: adt_theory.ml,v 1.14 2002/05/22 13:42:37 tews Exp $
  *
  *)
 
@@ -95,19 +95,31 @@ class ['class_type, 'member_type] ccsl_pvs_pre_adt_theory
 			 then (ccsl_final_theory_name c, args)
 			 else (ccsl_loose_theory_name c, args)
 			) :: accu
-		    | Spec_sig ->
-			let subst = make_substitution c#get_parameters args 
-			in
-			  (List.map 
-			     (fun (name,args) ->
-				(name, 
-				 substitute_arguments_types_only
-				   eq_types subst args)
-			     )
-			     c#get_iface_imports
+		    | Spec_sig -> 
+			if c#has_own_output
+			then		(* does have an own theory *)
+			  (ccsl_sig_def_theory_name c, args) :: accu
+			else		(* no own theory for c *)
+			  (if c#instanciate_importings && 
+			     adt#instanciate_importings
+			   then
+			     let subst = make_substitution 
+					   c#get_parameters args 
+			     in
+			       (List.map 
+				  (fun (name,args) ->
+				     (name, 
+				      substitute_arguments_types_only
+					eq_types subst args)
+				  )
+				  c#get_iface_imports
+			       )
+			   else
+			     List.map (fun (name,_) -> (name, [])) 
+			       c#get_iface_imports
 			  )
 			  @ accu
-		    | Spec_Spec -> (assert(false);raise Internal_error)
+		    | Spec_Spec -> assert(false)
 		 )
 		 adt#get_components
 		 []
@@ -184,18 +196,29 @@ class ['class_type, 'member_type] ccsl_isa_pre_adt_theory
 		 then (ccsl_final_theory_name c, args) :: accu
 		 else (ccsl_loose_theory_name c, args) :: accu
 	     | Spec_sig -> 
-		 let subst = make_substitution c#get_parameters args 
-		 in
-		   (List.map 
-		      (fun (name,args) ->
-			 (name, 
-			  substitute_arguments_types_only
-			    eq_types subst args)
-		      )
-		      c#get_iface_imports
+		 if c#has_own_output
+		 then			(* does have an own theory *)
+		   (ccsl_sig_def_theory_name c, args) :: accu
+		 else 			(* no own theory for c *)
+		   (if c#instanciate_importings && 
+		      adt#instanciate_importings
+		    then
+		      let subst = make_substitution c#get_parameters args 
+		      in
+			(List.map 
+			   (fun (name,args) ->
+			      (name, 
+			       substitute_arguments_types_only
+				 eq_types subst args)
+			   )
+			   c#get_iface_imports
+			)
+		    else
+		      List.map (fun (name,_) -> (name, [])) 
+			c#get_iface_imports
 		   )
 		   @ accu
-	     | Spec_Spec -> (assert(false);raise Internal_error)
+	     | Spec_Spec -> assert(false)
 	  )
 	  adt#get_components []
       in
@@ -625,7 +648,7 @@ class ['class_type, 'member_type] ccsl_isa_pre_adtutil_theory
 
 	| Product([]) 
 	| Product([_])
-	    -> assert(false); raise Internal_error
+	    -> assert(false)
 	| Product([t1;t2] as tl) ->
 	    let ssub_ns = sub_space sub_ns in
 	    let o_params,r_params,args = create_split_args ssub_ns csubst tl 
@@ -650,7 +673,7 @@ class ['class_type, 'member_type] ccsl_isa_pre_adtutil_theory
 	    let n = string_of_int( 9 )
 	    in let x = [t;t;t]
 	    in
-	    (assert(false); raise Internal_error)
+	    assert(false)
       in
       let red_funs,arg_types_list = 
 	List.split
@@ -862,14 +885,14 @@ class ['class_type, 'member_type] ccsl_pre_adt_every_theory
 			 (comp#get_model_type_argument args) :: args) 
 			:: accu
 		    | Spec_sig -> accu
-		    | Spec_Spec -> (assert(false); raise Internal_error)
+		    | Spec_Spec -> assert(false)
 	       )
 	       adt#get_components
 	       []
       )
 
 
-(* 		(* predicate lifting via primrec, does not work for nested adts *)
+(* 	(* predicate lifting via primrec, does not work for nested adts *)
  *     method private every =
  * 	    let sub_ns = sub_space ns in
  * 	    let type_param_ids = 
@@ -894,7 +917,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_every_theory
  * 		   | None, Some(name, _) -> (id, (None, build_term name))
  * 		   | Some (name1,_), Some(name2,_) ->
  * 		       (id, (build_term name1, build_term name2))
- * 		   | _ -> (assert(false); raise Internal_error)
+ * 		   | _ -> assert(false)
  * 		)
  * 		type_param_ids pred_ids_var
  * 	    in
@@ -1087,7 +1110,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_rellift_theory
 	       | Spec_class ->
 		   (ccsl_greatest_bibisim_theory_name c, []) :: accu
 	       | Spec_sig -> accu
-	       | Spec_Spec -> (assert(false);raise Internal_error)
+	       | Spec_Spec -> assert(false)
 	  )
 	  adt#get_components []
       in
@@ -1129,19 +1152,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_rellift_theory
       in
       let param_rel_lifting = 
 	var_terms_from_var_ids adt_param_ids rel_ids_var
-(* DELETE
- * 	 let build_term name = Some(Term(name,Always,[])) in
- * 	   List.map2
- * 	     (fun id -> function
- * 		| Some(name, _), None -> (id, (build_term name, None))
- * 		| None, Some(name, _) -> (id, (None, build_term name))
- * 		| Some (name1,_), Some(name2,_) ->
- * 		    (id, (build_term name1, build_term name2))
- * 		| _ -> (assert(false); raise Internal_error)
- * 	     )
- * 	     adt_param_ids rel_ids_var
- *)
-	 in
+      in
       let rel_ids = variance_flatten rel_ids_var in
       let rel_exprs = List.map (fun (id,t) -> Term(id,Always,[])) rel_ids
       in
@@ -1274,7 +1285,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_map_theory
 	       | Pos -> (th_name, from_args @ to_args) :: accu
 	       | Neg -> (th_name, to_args @ from_args) :: accu
 	       | Mixed -> do_v th_name (do_v th_name accu Pos) Neg
-	       | _ -> assert(false) ; raise Internal_error
+	       | _ -> assert(false)
 	     in
 	       match c#get_kind with
 		 | Spec_adt -> 
@@ -1284,7 +1295,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_map_theory
 	       | Spec_class ->
 		   do_v (ccsl_map_theory_name c) accu v
 	       | Spec_sig -> accu
-	       | Spec_Spec -> (assert(false);raise Internal_error)
+	       | Spec_Spec -> assert(false)
 	  )
 	  adt#get_components 
 	  []
@@ -1329,7 +1340,7 @@ class ['class_type, 'member_type] ccsl_pre_adt_map_theory
 	     | (None, _) as x -> x
 	     | (Some(neg_name, Function(dom,codom)), pos_id) ->
 		 (Some(neg_name, Function(codom, dom)), pos_id)
-	     | _ -> (assert(false); raise Internal_error)
+	     | _ -> assert(false)
 	  ) 
 	  pre_fun_ids_var
       in
@@ -1430,5 +1441,6 @@ class ccsl_adt_map_theory adt =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
