@@ -18,11 +18,11 @@
  *
  * Created 11.10.01 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 15 May 02 18:41:55 tews@ithif51>
  *
  * full invariants and bisimulations
  *
- * $Id: full_liftings_theory.ml,v 1.5 2002/01/24 14:43:59 tews Exp $
+ * $Id: full_liftings_theory.ml,v 1.7 2002/05/22 13:42:39 tews Exp $
  *
  *)
 
@@ -79,13 +79,16 @@ class ['class_type, 'member_type] ccsl_pre_full_invariant_theory
 
     method private do_imports =
       let do_ancestor = function
-	| Resolved_renaming (anc,args,_,_) -> 
+	| Resolved_renaming (anc,args,_,_,_) -> 
 	    (ccsl_full_invariant_theory_name anc, 
 	     self_argument :: args)
 						   (* no other stuff *)
 	| Unresolved_renaming _
-	| Resolved_ancestor _ 
-	| Unresolved_ancestor _ -> (assert(false); raise Internal_error)
+(* NO JAVA ANCESTORS
+ * 	   | Resolved_ancestor _ 
+ * 	   | Unresolved_ancestor _ 
+ *)
+	  -> assert(false)
       in
       let do_component (v,comp, args) accu =
 	match comp#get_kind with
@@ -97,7 +100,7 @@ class ['class_type, 'member_type] ccsl_pre_full_invariant_theory
 	      (ccsl_greatest_invariance_theory_name comp, 
 	       (comp#get_model_type_argument args) :: args) :: accu
 	  | Spec_sig -> accu
-	  | Spec_Spec -> (assert(false);raise Internal_error)
+	  | Spec_Spec -> assert(false)
       in
 	Import(
 	  (ccsl_interface_theory_name cl, self#simple_arguments)
@@ -113,7 +116,7 @@ class ['class_type, 'member_type] ccsl_pre_full_invariant_theory
       let positive_self_var = match cl#get_self_variance with
 	| Pair(-1,-1) -> Pair(-1,0)
 	| Unused -> Pos
-	| Unset -> (assert(false); raise Internal_error)
+	| Unset -> assert(false)
 	| v -> v
       in
       let var_p = create_ids_with_variance sub_ns
@@ -143,14 +146,14 @@ class ['class_type, 'member_type] ccsl_pre_full_invariant_theory
 	| Some(id,_), None -> Some( Term(id,Always,[])), None
 	| Some(id1,_), Some(id2,_) -> 
 	    Some( Term(id1,Always,[])), Some(Term(id2,Always,[]))
-	| _ -> assert(false); raise Internal_error
+	| _ -> assert(false)
       in
       let ancestor_list = 
         List.map (function
-                    | Resolved_renaming(oanc,args,_,ianc) -> oanc,args
+                    | Resolved_renaming(oanc,args,_,ianc,_) -> oanc,args
                                         (* no other stuff here *)
                     | Unresolved_renaming _
-                    | _ -> (assert(false); raise Internal_error)
+                    | _ -> assert(false)
                  ) cl#get_ancestors
       in
 						(* lifting for one ancestor *)
@@ -188,7 +191,7 @@ class ['class_type, 'member_type] ccsl_pre_full_invariant_theory
 		       -> [Term(p1,Always,[]); Term(p2,Always,[])]
 		   | Mixed, (Some(p1,_), Some(p2,_))
 		       -> [Term(p1,Always,[]); Term(p2,Always,[])]
-		   | _ -> assert(false); raise Internal_error
+		   | _ -> assert(false)
 		)),
 		Term(self_var,Always,[])))
       in
@@ -498,7 +501,7 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
 
     method private do_imports =
       let do_ancestor = function
-	| Resolved_renaming (anc,args,_,_) -> 
+	| Resolved_renaming (anc,args,_,_,_) -> 
 	    (ccsl_full_bibisim_theory_name anc, 
 	     self#self1_argument
 	     :: self#self2_argument
@@ -506,8 +509,11 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
 		 @ (ccsl_substitute_arguments self#subst2 args)))
 						   (* no other stuff *)
 	| Unresolved_renaming _
-	| Resolved_ancestor _ 
-	| Unresolved_ancestor _ -> (assert(false); raise Internal_error)
+(* NO JAVA ANCESTORS
+ * 	   | Resolved_ancestor _ 
+ * 	   | Unresolved_ancestor _ 
+ *)
+	  -> assert(false)
       in
       let do_component (v, comp, args) accu =
 	let subst_args1 = 
@@ -525,7 +531,7 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
 		 (comp#get_model_type_argument subst_args2) ::
 		 subst_args) :: accu
 	    | Spec_sig -> accu
-	    | Spec_Spec -> (assert(false);raise Internal_error)
+	    | Spec_Spec -> assert(false)
       in
 	Import(
 	  (ccsl_interface_theory_name cl, 
@@ -542,7 +548,7 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
       let positive_self_var = match cl#get_self_variance with
 	| Pair(-1,-1) -> Pair(-1,0)
 	| Unused -> Pos
-	| Unset -> (assert(false); raise Internal_error)
+	| Unset -> assert(false)
 	| v -> v
       in
       let var_r = create_ids_with_variance sub_ns 
@@ -579,14 +585,14 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
 	| Some(id,_), None -> Some( Term(id,Always,[])), None
 	| Some(id1,_), Some(id2,_) -> 
 	    Some( Term(id1,Always,[])), Some(Term(id2,Always,[]))
-	| _ -> assert(false); raise Internal_error
+	| _ -> assert(false)
       in
       let ancestor_list = 
         List.map (function
-                    | Resolved_renaming(oanc,args,_,ianc) -> oanc,args
+                    | Resolved_renaming(oanc,args,_,ianc,_) -> oanc,args
                                         (* no other stuff here *)
                     | Unresolved_renaming _
-                    | _ -> (assert(false); raise Internal_error)
+                    | _ -> assert(false)
                  ) cl#get_ancestors
       in
       let do_ancestor (anc, anc_arguments) =
@@ -630,7 +636,7 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
 		       -> [Term(r1,Always,[]); Term(r2,Always,[])]
 		   | Mixed, (Some(r1,_), Some(r2,_))
 		       -> [Term(r1,Always,[]); Term(r2,Always,[])]
-		   | _ -> assert(false); raise Internal_error
+		   | _ -> assert(false)
 		)),
               Tuple([Term(xname, Always,[]);
 		     Term(yname, Always,[])])))
@@ -694,7 +700,7 @@ class ['class_type, 'member_type] ccsl_pre_full_bibisim_theory
       let xid, yid = create_id_pairs sub_ns [Self] in
       let xname,yname = match xid,yid with
 	| [(x,_)],[(y,_)] -> x,y
-	| _ -> (assert(false); raise Internal_error)
+	| _ -> assert(false)
       in let xy_term = Tuple([Term(xname,Always,[]); Term(yname,Always,[])])
       in
       let type_param_ids = 
@@ -850,6 +856,7 @@ class ccsl_full_bibisim_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
 
