@@ -18,11 +18,11 @@
  *
  * Created 17.6.99 by Hendrik
  *
- * Time-stamp: <Tuesday 21 May 02 15:42:14 tews@ithif51>
+ * Time-stamp: <Monday 27 May 02 12:04:00 tews@ithif51>
  *
  * the ccsl interface theory
  *
- * $Id: interface_theory.ml,v 1.10 2002/05/22 13:42:40 tews Exp $
+ * $Id: interface_theory.ml,v 1.11 2002/05/30 13:10:30 tews Exp $
  *
  *)
 
@@ -33,6 +33,7 @@ open Name_space
 open Names
 open Top_names
 open Logic_util
+open Pvs_proof_util
 open Classtypes
 open Types_util
 open Theory_class
@@ -235,10 +236,6 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
 		    | Resolved_renaming (_,args,_,ianc,_) -> ianc
 				(* no other stuff *)
 		    | Unresolved_renaming _
-(* NO JAVA ANCESTORS
- * 		       | Resolved_ancestor _ 
- * 		       | Unresolved_ancestor _ 
- *)
 		      -> 
 			assert(false)
 		 ) in
@@ -380,11 +377,17 @@ class ['class_type, 'member_type] ccsl_pre_coadt_theory
 	 (if cl#has_feature HasFullRelLiftingFeature 
 	  then [ccsl_greatest_bibisim_theory_name cl,[]]
 	  else [])
-	 @
-	 (if cl#has_feature HasMorphismFeature
-	  then [ccsl_morphism_rewrite_theory_name cl,[];
-		ccsl_finality_theory_name cl,[]]
-	  else [])
+         @
+         (if cl#has_feature HasMorphismFeature
+          then 
+	    [
+(* via semantics now
+ * 		 ccsl_morphism_rewrite_theory_name cl,[];
+ *)
+              ccsl_finality_theory_name cl,[]
+	    ]
+          else [])
+
 	 @
 	 (if cl#has_feature FinalSemanticsFeature
 	  then [ccsl_final_props_theory_name cl,[]]
@@ -466,6 +469,25 @@ object (self : 'self)
 	))))
 	    
 	    
+  method private loose_decl = 
+    let tcc_proof = 
+	(* 
+	 * (LEMMA "loose_coalgebra")
+	 * (SKOSIMP* )
+	 * (INST?)
+	 *)
+      make_simple_proof [
+	lemma name_of_loose_coalg_axiom;
+	skosimp_star;
+	inst_question
+      ]
+    in
+      Proved(
+	Defnuninterpret(name_of_loose_coalgebra cl, [], self#lcoalg_type),
+	Named_proof(
+	  (name_of_loose_coalgebra cl) ^ "_TCC1",
+	  tcc_proof))
+
   method private alg_model_axiom =
     Axiom(
       name_of_loose_alg_axiom,
@@ -503,7 +525,7 @@ object (self : 'self)
     [Typevardecl (name_of_loose_type cl);
      Import( [ccsl_basic_theory_name cl, self#loose_args]);
      self#coalg_model_axiom;
-     Defnuninterpret(name_of_loose_coalgebra cl, [], self#lcoalg_type);
+     self#loose_decl;
     ]
     @
     if cl#has_constructors 
@@ -594,6 +616,25 @@ object (self : 'self)
 	))))
 	    
 	    
+  method private final_decl = 
+    let tcc_proof = 
+	(* 
+	 * (LEMMA "final_coalgebra")
+	 * (SKOSIMP* )
+	 * (INST?)
+	 *)
+      make_simple_proof [
+	lemma name_of_final_coalg_axiom;
+	skosimp_star;
+	inst_question
+      ]
+    in
+      Proved(
+	Defnuninterpret(name_of_final_coalgebra cl, [], self#fcoalg_type),
+	Named_proof(
+	  (name_of_final_coalgebra cl) ^ "_TCC1",
+	  tcc_proof))
+
   method private alg_model_axiom =
     Axiom(
       name_of_final_alg_axiom,
@@ -615,7 +656,7 @@ object (self : 'self)
     [Typevardecl (name_of_final_type cl);
      self#do_import;
      self#coalg_model_axiom;
-     Defnuninterpret(name_of_final_coalgebra cl, [], self#fcoalg_type);
+     self#final_decl
     ]
     @
     if cl#has_constructors 
