@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Sunday 19 May 02 13:34:40 tews@ithif56.inf.tu-dresden.de>
  *
  * predicate and relation lifting
  *
- * $Id: lifting.ml,v 1.17 2002/01/24 13:39:08 tews Exp $
+ * $Id: lifting.ml,v 1.19 2002/05/22 13:42:40 tews Exp $
  *
  *)
 
@@ -65,7 +65,7 @@ constraint 'mem = <get_domain : ('cl,'mem) top_pre_types;
 		   get_name : string;
 		   get_accessors : 'mem list;
 		   get_recognizer : 'mem;
-		   get_sort : 'mem top_member_sort;
+		   get_sort : 'mem top_pre_member_sort;
 		   ..>
 
 
@@ -90,7 +90,7 @@ constraint 'mem = <get_domain : ('cl,'mem) top_pre_types;
 		   get_name : string;
 		   get_accessors : 'mem list;
 		   get_recognizer : 'mem;
-		   get_sort : 'mem top_member_sort;
+		   get_sort : 'mem top_pre_member_sort;
 		   ..>
 
 (* 
@@ -109,7 +109,7 @@ let extend_param_rel_lifting param_list =
 	      | (id, (Some _, Some _)) as x -> x
 	      | (id, (None, Some p)) -> (id, (Some p, Some p))
 	      | (id, (Some p, None)) -> (id, (Some p, Some p))
-	      | _ -> assert(false); raise Internal_error
+	      | _ -> assert(false)
 	   )
     param_list
 
@@ -117,7 +117,7 @@ let extend_self_rel_lifting = function
   | (Some _, Some _) as x -> x 
   | (None, Some p) -> (Some p, Some p)
   | (Some p, None) -> (Some p, Some p)
-  | None,None -> assert(false); raise Internal_error
+  | None,None -> assert(false)
 
 
 
@@ -161,18 +161,25 @@ let rec pre_predlift
   let rec pred_lift_type name_space variance typ expr =
       match typ with 
         | Groundtype(id, args)   -> 
+	    if is_type_def id
+	    then			(* expand definition on the fly *)
+	      pred_lift_type name_space variance 
+		  (expand_type_def eq_types typ) expr
+
+	    else
 			(* For full predicate lifting we have to pass 
 			 * the predicate for the typeparameters
 			 * Therefore I generate an every here.
 			 *)
-	    let preds = pre_argument_list_predlift eq_types self_subst
-			  param_lifting self_lifting 
-			  name_space variance id.id_parameters args 
-	    in
-              Formula(
-	  	Application(
-		  Every(typ, preds),
-		  expr))
+	      let preds = pre_argument_list_predlift eq_types self_subst
+			    param_lifting self_lifting 
+			    name_space variance 
+			    (get_ground_type_parameters id) args 
+	      in
+		Formula(
+	  	  Application(
+		    Every(typ, preds),
+		    expr))
 	  
         | Class(cl, args) -> 
 	    if !Global.inline_liftings &&
@@ -282,7 +289,7 @@ let rec pre_predlift
 	| FreeTypeVariable _
         | IFace _
         | Array _         
-    	| SmartFunction _ -> (assert(false);raise Internal_error)
+    	| SmartFunction _ -> assert(false)
 
   in
     pred_lift_type top_name_space variance typ expression
@@ -333,7 +340,7 @@ and pre_argument_list_predlift
 	     (non_trivial_pred_lift parameter_lifting full_self_lifting 
 		variance typ) 
 	     :: accu
-	 | _ -> (assert(false);raise Internal_error)
+	 | _ -> assert(false)
       )
 				(* produce a list of type * variance pairs *)
       (List.combine
@@ -854,27 +861,29 @@ let rec pre_rellift
   let rec rel_lift_type name_space variance typ expr1 expr2 =
     match typ with 
       | Groundtype(id, args)   -> 
+	  if is_type_def id
+	  then				(* expand definition on the fly *)
+	    rel_lift_type name_space variance
+	      (expand_type_def eq_types typ) expr1 expr2
+
+	  else
 	        (* !!! same discussion as for predicate lifting !!! 
 		 * 
 		 * For full relation lifting we have to pass 
 		 * the predicate for the typeparameters
 		 * Therefore I generate an every here.
 		 *)
-(* 	     let _ = assert( 
- * 	       List.for_all (function 
- * 			     | TypeArgument(typ) -> constant_type param_p typ
- * 			    )
- * 				  args) in
- *)
-	  let rels = pre_argument_list_rellift eq_types
-		       self1_subst self2_subst
-		       param_rel_lifting self_lifting 
-		       name_space variance id.id_parameters args 
-	  in
-	    Formula(
-	      Application(
-		RelEvery(typ, rels),
-		Tuple([expr1;expr2])))
+	    let rels = pre_argument_list_rellift eq_types
+			 self1_subst self2_subst
+			 param_rel_lifting self_lifting 
+			 name_space variance
+			 (get_ground_type_parameters id)
+			 args 
+	    in
+	      Formula(
+		Application(
+		  RelEvery(typ, rels),
+		  Tuple([expr1;expr2])))
 
       | Class(cl, args) -> 
 	  if !Global.inline_liftings && 
@@ -936,7 +945,7 @@ let rec pre_rellift
 				(* self1 and self2 have different types 
 				 * so equality does not work here
 				 *)
-		  | _ -> (assert(false); raise Internal_error)
+		  | _ -> assert(false)
 	       )
 
       | Bool -> Equal(expr1, expr2)
@@ -1003,7 +1012,7 @@ let rec pre_rellift
       | Predtype _
       | IFace _
       | Array _ 	      
-      | SmartFunction _ -> (assert(false);raise Internal_error)
+      | SmartFunction _ -> assert(false)
   in
     rel_lift_type top_name_space variance typ expr1 expr2
   
@@ -1061,7 +1070,7 @@ and pre_argument_list_rellift
              (non_trivial_rel_lift parameter_rel_lifting full_self_lifting 
 		variance typ) 
              :: accu
-         | _ -> (assert(false);raise Internal_error)
+         | _ -> assert(false)
       )
                                 (* produce a list of type * variance pairs *)
       (List.combine
@@ -1410,7 +1419,7 @@ let adt_rel_lifting_substitutions adt type_param1 type_param2 =
 		  * this method exists only for theories, not for ifaces 
 		  *)
       (Adt(adt, Always, 
-	  arguments_from_parameters(adt#get_parameters))) in
+	  arguments_from_parameters adt_type_parameters)) in
   let self2_subst = 
     ((Carrier, adt2_type) :: self2_pre_subst) in
   let self1_subst = 
@@ -1640,7 +1649,7 @@ let single_step_type2parameter dom_type codom_type acc_fn =
       | FreeTypeVariable _
       | Record _
       | Array _
-      | Predtype _ -> (assert(false);raise Internal_error)
+      | Predtype _ -> assert(false)
   in 
   let augment_with_domain (types,name,accessors,succ_fun) =
     match dom_type with
@@ -1662,7 +1671,7 @@ let single_step_type2parameter dom_type codom_type acc_fn =
 		 )
 	      )
 	    )
-      | _ -> (assert(false); raise Internal_error)
+      | _ -> assert(false)
   in
     List.map
       augment_with_domain
@@ -1696,7 +1705,7 @@ let step_constructor_declarations ns cl =
     List.map 
       (fun t -> match t with
 	 | Term(s,_,_) -> s
-	 | _ -> (assert(false);raise(Internal_error))
+	 | _ -> assert(false)
       )
       tl
   in 
@@ -1707,12 +1716,10 @@ let step_constructor_declarations ns cl =
     let dom = match m#get_domain with
       | Self -> Product([])
       | Product(Self::tl) -> Product(tl)
-      | _ -> (let debug_dm = m#get_domain in 
-	      let debug_name = m#get_name in
-		assert(false); raise(Internal_error))
+      | _ -> assert(false)
     in
-    List.map (triple_for_path m#get_name) 
-      (single_step_type2parameter dom m#get_codomain accessors)
+      List.map (triple_for_path m#get_name) 
+	(single_step_type2parameter dom m#get_codomain accessors)
   in
   let triples_for_methods ml =
     List.fold_right
@@ -1758,7 +1765,7 @@ let admissible_cases_for_method ns mem check_list_term =
     match mem#get_domain with
       | Self -> Product([])
       | Product(Self::tl) -> Product(tl)
-      | _ -> (assert(false); raise Internal_error)
+      | _ -> assert(false)
   in
     List.map
       case_for_path
@@ -1787,7 +1794,7 @@ let transition_cases_for_method ns mem coalg_term x_term =
   let dom = match mem#get_domain with
     | Self -> Product([])
     | Product(Self::tl) -> Product(tl)
-    | _ -> (assert(false); raise Internal_error)
+    | _ -> assert(false)
   in
     List.map
       (fun (ty,st,accl,fn) ->
@@ -1808,4 +1815,5 @@ let transition_cases_for_method ns mem coalg_term x_term =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
