@@ -18,11 +18,11 @@
  *
  * * Created 15.1.99 by Hendrik
  *
- * Time-stamp: <Monday 20 May 02 21:19:42 tews@ithif56.inf.tu-dresden.de>
+ * Time-stamp: <Tuesday 28 May 02 16:23:54 tews@ithif51>
  *
  * main compiler driver
  *
- * $Id: main.ml,v 1.16 2002/05/22 13:42:41 tews Exp $
+ * $Id: main.ml,v 1.17 2002/05/30 13:10:30 tews Exp $
  *
  *)
 
@@ -230,6 +230,9 @@ let options =
     ("-no-inline-lifting",
      Arg.Clear (inline_liftings),
      "Do not inline liftings for non-recursive classes and adt's");
+    ("-dependent-assertions",
+     Arg.Set(dependent_assertions),
+     "Make each assertions dependent on the previous ones");
     ("-fixedpointlib",
      Arg.String (set_pvs_ccsl_lib_location),
      "Directory for the fixedpoint library (PVS only)")
