@@ -18,11 +18,11 @@
  *
  * Created 16.03.00 by Jan
  *
- * Time-stamp: <Wednesday 15 May 02 19:27:35 tews@ithif51>
+ * Time-stamp: <Sunday 26 May 02 19:16:38 tews@ithif56.inf.tu-dresden.de>
  *
  * Coalgebra morphism theories
  *
- * $Id: morphism_theory.ml,v 1.10 2002/05/22 13:42:42 tews Exp $
+ * $Id: morphism_theory.ml,v 1.11 2002/05/30 13:10:30 tews Exp $
  *
  *)
 
@@ -32,10 +32,11 @@ open Top_variant_types
 open Name_space
 open Top_names
 open Names
+open Logic_util
+open Pvs_proof_util
 open Classtypes
 open Types_util
 open Theory_class
-open Logic_util
 open Morphism;;
 
 (***********************************************************************
@@ -455,8 +456,11 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
 
     method private imports =
       Import(
-	[((ccsl_morphism_rewrite_theory_name cl),
-	  self#double_self_arguments);
+	[
+(* via Semantics theory now
+ * 	     ((ccsl_morphism_rewrite_theory_name cl),
+ * 	     self#double_self_arguments);
+ *)
 	 ((ccsl_basic_theory_name cl),
 	  self#self1_argument :: orig_arguments);
 	 ((ccsl_basic_theory_name cl),
@@ -566,6 +570,28 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
 
 
     method private struct_coreduce_def = 
+	    (* (""
+	     *  (SKOLEM-TYPEPRED)
+	     *  (EXPAND* "nonempty?" "empty?" "member" "Project_final?"
+	     *   "Project_morphism_exists?")
+	     *  (FLATTEN)
+	     *  (HIDE-ALL-BUT (-3 -5))
+	     *  (INST?)
+	     *  (SKOSIMP)
+	     *  (INST?))
+             *)
+      let tcc_proof =
+	make_simple_proof[
+	  skolem_typepred;
+	  expand_star ["nonempty?"; "empty?"; "member"; (name_of_finality cl);
+		       (name_of_finality_existence cl)];
+	  flatten;
+	  hide_all_but [-3; -5];
+	  inst_question;
+	  skosimp;
+	  inst_question
+	]
+      in
       let sub_ns = sub_space ns in
       let type_of_morphism = 	  
 	Function(
@@ -573,7 +599,7 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
 	  BoundTypeVariable(self2)
       	)
       in let f = create_one_id sub_ns type_of_morphism 
-      in
+      in let decl = 
       Defn(
 	name_of_class_struct_coreduce cl,
 	[[Undeclared(name_of_coalgebra2,
@@ -592,7 +618,11 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
 		Tuple([coalgebra1_term;
 		       coalgebra2_term])),
 	      Term(f,Never,[])))
-	))		     
+	))
+      in
+	Proved(decl,
+	       Named_proof((name_of_class_struct_coreduce cl) ^ "_TCC1",
+			   tcc_proof))
 		     
 
     method private coreduce_def = 
@@ -605,6 +635,18 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
       in let f = create_one_id sub_ns type_of_morphism in
       let mt = create_id_with_preference sub_ns "m_tuple" "p" 
       in
+	    (* (""
+	     *  (SKOLEM-TYPEPRED)
+	     *  (EXPAND "ProjectMethodAssert?")
+	     *  (REPLACE-ETA "m_tuple!1"))
+	     *)
+      let tcc_proof = 
+	make_simple_proof [
+	  skolem_typepred;
+	  expand (name_of_method_assert cl);
+	  replace_eta (pvs_skolem mt)
+	]
+      in let decl = 
       Defn(
 	name_of_class_coreduce cl,
 	[[Undeclared(name_of_coalgebra2,
@@ -623,6 +665,10 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
 	    Term(name_of_struct_of cl, Always,[]),
 	    Term(mt,Always,[]))
 	))
+      in 
+	Proved(decl,
+	       Named_proof((name_of_class_coreduce cl) ^ "_TCC1",
+			   tcc_proof))
 
 
     method private finality_bisim_eq_lemma =
