@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Tuesday 21 May 02 18:42:39 tews@ithif51>
+ * Time-stamp: <Wednesday 22 May 02 18:23:45 tews@ithif51>
  *
  * strings for ccsl translation
  *
- * $Id: names.ml,v 1.9 2002/05/22 13:42:42 tews Exp $
+ * $Id: names.ml,v 1.10 2002/05/30 13:10:30 tews Exp $
  *
  *)
 
@@ -211,11 +211,12 @@ let name_of_struct_of cl =
     | Isa_mode -> cl#get_name ^ "_struct"
   
 
-
-let name_of_method_tag m = 
+let name_of_method_tag_string name = 
   match !output_mode with
-    | Pvs_mode -> m#get_name
-    | Isa_mode -> m#get_name ^ "_tag"
+    | Pvs_mode -> name
+    | Isa_mode -> name ^ "_tag"
+
+let name_of_method_tag m = name_of_method_tag_string m#get_name
 
 let name_of_method_tag_recognizer m = 
   match !output_mode with
