@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Tuesday 21 May 02 18:10:16 tews@ithif51>
+ * Time-stamp: <Wednesday 22 May 02 18:21:53 tews@ithif51>
  *
  * strings for ccsl translation
  *
- * $Id: names.mli,v 1.9 2002/05/22 13:42:42 tews Exp $
+ * $Id: names.mli,v 1.10 2002/05/30 13:10:31 tews Exp $
  *
  *)
 
@@ -173,6 +173,8 @@ val super_access_method : <get_name : string; .. > -> string
 
 val method_label : <get_name : string; .. > -> string
 
+val name_of_method_tag_string : string -> string
+
 val name_of_method_tag : <get_name : string; .. > -> string
 
 val name_of_method_tag_recognizer : <get_name : string; .. > -> string
