@@ -18,11 +18,11 @@
  *
  * Created 3.11.99 by Hendrik
  *
- * Time-stamp: <Wednesday 15 May 02 18:42:48 tews@ithif51>
+ * Time-stamp: <Tuesday 28 May 02 17:33:18 tews@ithif51>
  *
  * Semantics
  *
- * $Id: semantics_theory.ml,v 1.13 2002/05/22 13:42:43 tews Exp $
+ * $Id: semantics_theory.ml,v 1.14 2002/05/30 13:10:31 tews Exp $
  *
  *)
 
@@ -122,9 +122,12 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
      * applies term to string
      * nameing_function is a function, that gives the name for 
      * the declaration
+     * ass is the assertion itself
+     * previous_option is an assertion option, it makes the assertion 
+     * dependent on the previous one (for -dependent-assertions)
      *)
     method private do_ass_or_crea nameing_function 
-                                  let_trick_list ass base_decl = 
+                             let_trick_list ass base_decl previous_option = 
       let base_typ = snd(base_decl) in
       let decl_from_rec id = 
 	id.id_token.token_name, id.id_type  in
@@ -174,13 +177,7 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 			       	  term)))
 	       	  (filter_let_trick_list text_formula)),
 	       Expression(ConstantPredicate(text_formula))))
-      in
-      	Defn( nameing_function ass.assertion_name.token_name,
-	      [[Declared(name_of_coalgebra, self#coalgebra_type)]],
-	      Function(base_typ, Bool),
-	      Abstraction(
-		[base_decl],
-		Expression(
+      in let inner_ex =
 		  Forall(
 		    List.map decl_from_rec ass.free_variables,
 		    match ass.assertion_formula with
@@ -192,7 +189,28 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 			  assert(!output_mode = Pvs_mode); let_trick f
 		      | Isa_String f -> 
 			  assert(!output_mode = Isa_mode); let_trick f
-		  ))))
+		  )
+      in
+      	Defn( nameing_function ass.assertion_name.token_name,
+	      [[Declared(name_of_coalgebra, self#coalgebra_type)]],
+	      Function(base_typ, Bool),
+	      Abstraction(
+		[base_decl],
+		Expression(
+		  match previous_option with 
+		    | None -> inner_ex
+		    | Some pre_ass ->
+			And[
+			  Formula(
+			    Application(
+			      Application(
+				Term(nameing_function 
+				       pre_ass.assertion_name.token_name,
+				       Always, []),
+				coalgebra_term),
+			      Term(fst base_decl, Always, [])));
+			  inner_ex]
+		)))
 	      
     method private do_all_assertions =
       let let_trick_list =
@@ -220,6 +238,8 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 		      Function(m#get_domain,m#get_codomain),
 		      coalgebra_term))
 	   cl#get_all_actions)
+      in
+      let previous_ass = ref None 
       in
       	(List.map 
 	   (fun ass -> 
@@ -228,9 +248,12 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 	      	  | None -> assert(false)
 	      	  | Some id -> (id.id_token.token_name,
 			      	id.id_type)
+	      in let decl = self#do_ass_or_crea name_of_assertion_predicate
+			      let_trick_list ass base_decl !previous_ass;
 	      in
-	      	self#do_ass_or_crea name_of_assertion_predicate
-		  let_trick_list ass base_decl
+		if !dependent_assertions then
+		  previous_ass := Some ass;
+		decl
 	   )
 	   cl#get_assertions)
 
@@ -293,7 +316,7 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 	      let base_decl = (name_of_algebra, self#algebra_type)
 	      in
 	      	self#do_ass_or_crea name_of_creation_predicate
-		  let_trick_list crea base_decl
+		  let_trick_list crea base_decl None
 	   )
 	   cl#get_creations)
 	
@@ -461,42 +484,70 @@ class ['class_type, 'member_type] ccsl_pre_basic_theory
       Import [ccsl_semantics_theory_name cl, self#simple_arguments]
 
     method private do_inherited_assert_lemmas = 
+         (* First built the proof. The proof is identical for all
+          * super classes.
+	  *)
       let proof = 
-	make_simple_proof
+	make_simple_proof	(* make a simple proof without branching *)
 	  [
-	    skosimp_star;
-	    expand (name_of_assert cl);
-	    pvs_assert;
+	    skosimp_star;			(* (SKOSIMP* ) *)
+	    expand (name_of_assert cl);		(* (EXPAND "<class>Assert?") *)
+	    pvs_assert;				(* (ASSERT) *)
 	  ]
       in
+	 (* The following function lemma computes the inherit assert 
+	  * lemma for its argument ianc, a super class of the current 
+	  * class
+	  *)
       let lemma ianc =
-	Lemma(
-	  name_of_inherited_assert_lemma ianc,
-	  Forall(
+	Lemma(					(* its a lemma (not a type) *)
+	  name_of_inherited_assert_lemma ianc,	(* get its name *)
+	  Forall(				(* forall coalgebras c *)
 	    [(name_of_coalgebra, self#coalgebra_type)],
 	    Implies(
-	      self#assert_coalgebra_hypothesis,
+	      self#assert_coalgebra_hypothesis,	(* Assert?(c) IMPLIES *)
 	      Formula(
-		Application(
+		Application(			(* Assert?(super(c)) *)
 		  Term(name_of_assert ianc, Always, []),
 		  Application(Term(super_access_method(ianc),
 				   Always,[]), 
 			      coalgebra_term)
 		)))))
       in
-      List.map
-	(fun ianc -> 
-	   Proved(
+      List.map				(* iterate over all direct ancestors *)
+	(fun ianc ->				(* iteration function *)
+	   Proved(			(* Pack lemma together with proof *)
 	     lemma ianc,
 	     Anon_proof proof))
-	cl#get_resolved_ancestors
+	cl#get_resolved_ancestors		(* get all direct ancestors *)
 
-    method private assertion_lemma_proof ass number_of_ass = 
+
+    method private assertion_lemma_proof ass number_of_ass previous_opt = 
       let ancestor_number = List.length cl#get_ancestors in
       let self_var = 
 	match ass.self_variable with
 	  | None -> assert(false)
 	  | Some id -> id.id_token.token_name
+      in let case_if_previous = match previous_opt with
+	| None -> []
+	| Some ass -> 
+	    [case (Pvs_pretty.string_of_pvs_top_expression(
+		     Application(
+		       Application(
+			 Term(name_of_assertion_predicate 
+				ass.assertion_name.token_name, Always, []),
+			 Term(pvs_skolem name_of_coalgebra, Always, [])),
+		       Term(pvs_skolem self_var, Always, []))))
+	    ]
+      in let other_cases = match previous_opt with
+	| None -> []
+	| Some _ -> [make_simple_proof [
+		       flatten_antecedent;
+		       pvs_assert
+		     ]]
+      in let assert_if_previous = match previous_opt with
+	| None -> []
+	| Some _ -> [pvs_assert]
       in
 	PTree( 
 	  [ 
@@ -506,20 +557,33 @@ class ['class_type, 'member_type] ccsl_pre_basic_theory
 		       Term(name_of_assert cl, Always, []),
 		       Term(pvs_skolem name_of_coalgebra, Always, []))))
 	  ],
-	  [make_simple_proof
-	     [
-	       pvs_assert;
-	       expand (name_of_assert cl);
-	       flatten_antecedent;
-	       hide_all_but [-(ancestor_number +1); 1];
-	       inst_val (-1) (pvs_skolem self_var);
-	       flatten_antecedent;
-	       hide_all_but [- number_of_ass; 1];
-	       expand (name_of_assertion_predicate 
-			 ass.assertion_name.token_name);
-	       inst_question;
-	       pvs_assert;
-	     ];
+	  [PTree
+	     ([
+		pvs_assert;
+		expand (name_of_assert cl);
+		flatten_antecedent;
+		hide_all_but [-(ancestor_number +1); 1];
+		inst_val (-1) (pvs_skolem self_var);
+		flatten_antecedent;
+		hide_all_but [- number_of_ass; 1];
+		expand (name_of_assertion_predicate 
+			  ass.assertion_name.token_name);
+	      ]
+	      @ (
+		case_if_previous
+	      ),
+	      (make_simple_proof
+		 (assert_if_previous
+		  @
+		  [
+		    inst_question;
+		    pvs_assert;
+		  ]
+		 )
+	      ) :: (
+		other_cases
+	      )
+	     );
 	   make_simple_proof
 	     [pvs_assert]
 	  ])
@@ -582,14 +646,18 @@ class ['class_type, 'member_type] ccsl_pre_basic_theory
 	  Anon_proof proof)
 
     method private do_assertion_lemmas = 
-      let ass_count = ref 1 
+      let ass_count = ref 1 in
+      let previous_ass = ref None 
       in
 	List.map 
 	  (fun ass -> 
 	     let lemma = self#do_assertion_lemma ass 
-			 (self#assertion_lemma_proof ass !ass_count)
+			 (self#assertion_lemma_proof ass !ass_count
+			    !previous_ass)
 	     in
 	       incr ass_count;
+	       if !dependent_assertions then
+		 previous_ass := Some ass;
 	       lemma
 	  )
 	  cl#get_assertions
