@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Sunday 19 May 02 11:26:47 tews@ithif56.inf.tu-dresden.de>
+ * Time-stamp: <Sunday 26 May 02 11:43:59 tews@ithif56.inf.tu-dresden.de>
  *
  * Variance checking pass for CCSL
  *
- * $Id: variance.ml,v 1.7 2002/05/22 13:42:44 tews Exp $
+ * $Id: variance.ml,v 1.8 2002/05/30 13:10:31 tews Exp $
  *
  *)
 
@@ -474,10 +474,11 @@ let do_sig si =
   in
   let global_param_ids = 
     List.map (function TypeParameter id -> id) si#get_parameters 
-  in let _ =
+  in
+     (* require global variance annotations if type declarations are present *)
+  let _ =
       if tdecl <> []
       then
-         (* there is a true type declaration ==> require variance annotation *)
 	List.iter 
 	  (fun id -> (if (id.id_variance = Unset)
 		      then
@@ -486,7 +487,7 @@ let do_sig si =
 	  global_param_ids
   in
 
-  let _ =	(* require variance annotions in all type declarations *)
+  let _ =	(* require local variance annotions in all type declarations *)
     List.iter
       (fun tdecl_id -> 
 	 List.iter (function TypeParameter id ->
@@ -534,7 +535,17 @@ let do_sig si =
 		  else
 		    variance_mismatch_error id uvar
 	       )
-      global_param_ids global_user_vars
+      global_param_ids global_user_vars;
+
+					(* set local variance in definitions *)
+    List.iter (fun m ->
+		 List.iter (function TypeParameter id ->
+			      if id.id_variance = Unset then
+				id.id_variance <- Unused
+			   )
+		   m#get_local_parameters
+	      )
+      si#get_members
 
 
 let variance_ast = function
