@@ -18,11 +18,11 @@
  *
  * Created 15.1.99 by Hendrik
  *
- * Time-stamp: <Wednesday 22 May 02 11:02:43 tews@ithif51>
+ * Time-stamp: <Tuesday 28 May 02 18:15:38 tews@ithif51>
  *
  * global control variables of the loop compiler
  *
- * $Id: global.ml,v 1.15 2002/05/22 13:42:45 tews Exp $
+ * $Id: global.ml,v 1.16 2002/05/30 13:10:31 tews Exp $
  *
  *)
 
@@ -39,7 +39,7 @@ open Top_variant_types
  *)
 
 
-let compiler_version = "CCSL Compiler Version 2.1.3"
+let compiler_version = "CCSL Compiler Version 2.1.4"
 
 
 (***********************************************************************
@@ -63,38 +63,12 @@ let output_mode = ref (Pvs_mode : output_mode)
 let parse_prelude = ref true
 
 
-    (* if true enable the java2 option in the java compiler
-     * set in main.ml, via -java2 option
-     *)
-let java2_mode = ref false
-
-
-    (* if true enable the jml option in the java compiler
-     * set in main.ml, via -jml option
-     *)
-let jml_mode = ref false
-
-
     (* do things, a normal user should never see
      * set via -expert
      *)
 let expert_mode = ref false
 
 
-    (* generate method bodies without local variables
-     * set via -hoare
-     *)
-let hoare_mode = ref false
-
-
-    (* don't use the fixed point definition for method calls,
-     * but the direct rewrite instead. This simplifies verification
-     * when there are no (mutually) recursive method definitions.
-     * set via -no-recursion
-     *)
-let no_java_recursion = ref false
-
-
     (* if <> "" generate all output in this file
      * set in main.ml, via -o option
      *)
@@ -127,63 +101,6 @@ let get_target_directory () =
   !target_directory
 
 
-(***********************************************************************
- ***********************************************************************
- *
- * debug level section
- *
- *)
-
-    (* debug_level_var: controls amount of status information
-     * 1 : identical to -v (verbose)
-     * 2 : print information about lexer
-     * 4 : print information about parser
-     * 8 : print information about resolution
-     * 16 : print information about inheritance
-     * 32 : print information about typecheck
-     * 64 : print information about syntax tree
-     * 128 : print information about symbol table
-     * 256 : apply everything also to the prelude
-     * 512 : print information about unification
-     * 1024 : print assertions directly after parsing
-     * 2048 : variance pass
-     * 4096 : feature pass
-     * set in main.ml, via -D [int] option
-     *)
-let _VERBOSE = 1
-let _DEBUG_LEXER = 2
-let _DEBUG_PARSER = 4
-let _DEBUG_RESOLUTION = 8
-let _DEBUG_INHERITANCE = 16
-let _DEBUG_TYPECHECK = 32
-let _DEBUG_DUMP_SYNTAX_TREE = 64
-let _DEBUG_DUMP_SYMBOL_TABLE = 128
-let _DEBUG_PRELUDE = 256
-let _DEBUG_UNIFICATION = 512
-let _DEBUG_PARSERCHECK = 1024
-let _DEBUG_VARIANCE = 2048
-let _DEBUG_FEATURES = 4096
-
-let debug_level_var = ref 0
-
-    (* access functions for debug_level_var
-     *)
-let add_debug_level level =
-  if level > 0
-  then
-    debug_level_var := !debug_level_var lor level
-  else
-    debug_level_var := 0
-
-
-let get_debug_level () =
-  !debug_level_var
-
-
-let debug_level pass =
-  (!debug_level_var land pass) <> 0
-
-
     (* if false, do not generate any theories
      * unset in main.ml, via option -prf
      *)
@@ -266,9 +183,80 @@ let deselected_classes = ref ([] : string list)
 let optimize_expressions = ref true
 
 
+
+    (*
+     * enable inlining of liftings of constant classes and data types
+     *
+     * unset via -no-inline-lifting
+     *)
 let inline_liftings = ref true
 
 
+    (*
+     * enable chained assertion predicate
+     *
+     * set via -dependent-assertions
+     *)
+let dependent_assertions = ref false
+
+
+(***********************************************************************
+ ***********************************************************************
+ *
+ * debug level section
+ *
+ *)
+
+    (* debug_level_var: controls amount of status information
+     * 1 : identical to -v (verbose)
+     * 2 : print information about lexer
+     * 4 : print information about parser
+     * 8 : print information about resolution
+     * 16 : print information about inheritance
+     * 32 : print information about typecheck
+     * 64 : print information about syntax tree
+     * 128 : print information about symbol table
+     * 256 : apply everything also to the prelude
+     * 512 : print information about unification
+     * 1024 : print assertions directly after parsing
+     * 2048 : variance pass
+     * 4096 : feature pass
+     * set in main.ml, via -D [int] option
+     *)
+let _VERBOSE = 1
+let _DEBUG_LEXER = 2
+let _DEBUG_PARSER = 4
+let _DEBUG_RESOLUTION = 8
+let _DEBUG_INHERITANCE = 16
+let _DEBUG_TYPECHECK = 32
+let _DEBUG_DUMP_SYNTAX_TREE = 64
+let _DEBUG_DUMP_SYMBOL_TABLE = 128
+let _DEBUG_PRELUDE = 256
+let _DEBUG_UNIFICATION = 512
+let _DEBUG_PARSERCHECK = 1024
+let _DEBUG_VARIANCE = 2048
+let _DEBUG_FEATURES = 4096
+
+let debug_level_var = ref 0
+
+    (* access functions for debug_level_var
+     *)
+let add_debug_level level =
+  if level > 0
+  then
+    debug_level_var := !debug_level_var lor level
+  else
+    debug_level_var := 0
+
+
+let get_debug_level () =
+  !debug_level_var
+
+
+let debug_level pass =
+  (!debug_level_var land pass) <> 0
+
+
 (***********************************************************************
  ***********************************************************************
  *
