@@ -18,11 +18,11 @@
  *
  * Created 15.1.99 by Hendrik
  *
- * Time-stamp: <Wednesday 8 May 02 11:26:30 tews@ithif51>
+ * Time-stamp: <Tuesday 28 May 02 18:15:45 tews@ithif51>
  *
  * global control variables of the loop compiler
  *
- * $Id: global.mli,v 1.10 2002/05/22 13:42:45 tews Exp $
+ * $Id: global.mli,v 1.11 2002/05/30 13:10:31 tews Exp $
  *
  *)
 
@@ -60,18 +60,6 @@ val output_mode : output_mode ref
 (* val parse_prelude : bool ref *)
 
 
-    (* if true enable the java2 option in the java compiler
-     * set in main.ml, via -java2 option
-     *)
-val java2_mode : bool ref
-
-
-    (* if true enable the jml option in the java compiler
-     * set in main.ml, via -jml option
-     *)
-val jml_mode : bool ref
-
-
     (* 
      * do things, a normal user should never see
      * set via -expert
@@ -79,20 +67,6 @@ val jml_mode : bool ref
 val expert_mode : bool ref
 
 
-    (* generate method bodies without local variables
-     * set via -hoare
-     *)
-val hoare_mode : bool ref
-
-
-    (* don't use the fixed point definition for method calls,
-     * but the direct rewrite instead. This simplifies verification
-     * when there are no (mutually) recursive method definitions.
-     * set via -no-recursion
-     *)
-val no_java_recursion : bool ref
-
-
     (* if <> "" generate all output in this file
      * set in main.ml, via -o option
      *)
@@ -115,56 +89,6 @@ val get_target_directory : unit -> string
 val set_target_directory : string -> unit
 
 
-(***********************************************************************
- ***********************************************************************
- *
- * debug level section
- *
- *)
-
-
-    (* debug_level_var: controls amount of status information
-     * 1 : identical to -v (verbose)
-     * 2 : print information about lexer
-     * 4 : print information about parser
-     * 8 : print information about resolution
-     * 16 : print information about inheritance
-     * 32 : print information about typecheck
-     * 64 : print information about syntax tree
-     * 128 : print information about symbol table
-     * 512 : print information about unification
-     * 1024 : print assertion/creation condition right after parsing
-     * set in main.ml, via -D [int] option
-     *)
-val _VERBOSE :                 int 
-val _DEBUG_LEXER : 	       int 
-val _DEBUG_PARSER : 	       int 
-val _DEBUG_RESOLUTION :        int 
-val _DEBUG_INHERITANCE :       int 
-val _DEBUG_TYPECHECK : 	       int 
-val _DEBUG_DUMP_SYNTAX_TREE :  int 
-val _DEBUG_DUMP_SYMBOL_TABLE : int 
-val _DEBUG_PRELUDE : 	       int
-val _DEBUG_UNIFICATION :       int
-val _DEBUG_PARSERCHECK :       int
-val _DEBUG_VARIANCE :	       int
-val _DEBUG_FEATURES : 	       int
-
-    (* access functions for debug_level_var
-     *)
-
-    (* lor more bits into debug_level_var
-     * argument of 0 resets debug_level_var
-     *)
-val add_debug_level : int -> unit
-
-
-val get_debug_level : unit -> int
-
-
-val debug_level : int -> bool
-
-
     (* if false, do not generate any theories
      * unset in main.ml, via option -prf
      *)
@@ -243,8 +167,71 @@ val selected_classes : string list ref
 val optimize_expressions : bool ref
 
 
+    (*
+     * enable inlining of liftings of constant classes and data types
+     *
+     * unset via -no-inline-lifting
+     *)
 val inline_liftings : bool ref
 
+
+    (*
+     * enable chained assertion predicate
+     *
+     * set via -dependent-assertions
+     *)
+val dependent_assertions : bool ref
+
+(***********************************************************************
+ ***********************************************************************
+ *
+ * debug level section
+ *
+ *)
+
+
+    (* debug_level_var: controls amount of status information
+     * 1 : identical to -v (verbose)
+     * 2 : print information about lexer
+     * 4 : print information about parser
+     * 8 : print information about resolution
+     * 16 : print information about inheritance
+     * 32 : print information about typecheck
+     * 64 : print information about syntax tree
+     * 128 : print information about symbol table
+     * 512 : print information about unification
+     * 1024 : print assertion/creation condition right after parsing
+     * set in main.ml, via -D [int] option
+     *)
+val _VERBOSE :                 int 
+val _DEBUG_LEXER : 	       int 
+val _DEBUG_PARSER : 	       int 
+val _DEBUG_RESOLUTION :        int 
+val _DEBUG_INHERITANCE :       int 
+val _DEBUG_TYPECHECK : 	       int 
+val _DEBUG_DUMP_SYNTAX_TREE :  int 
+val _DEBUG_DUMP_SYMBOL_TABLE : int 
+val _DEBUG_PRELUDE : 	       int
+val _DEBUG_UNIFICATION :       int
+val _DEBUG_PARSERCHECK :       int
+val _DEBUG_VARIANCE :	       int
+val _DEBUG_FEATURES : 	       int
+
+    (* access functions for debug_level_var
+     *)
+
+    (* lor more bits into debug_level_var
+     * argument of 0 resets debug_level_var
+     *)
+val add_debug_level : int -> unit
+
+
+val get_debug_level : unit -> int
+
+
+val debug_level : int -> bool
+
+
 (***********************************************************************
  ***********************************************************************
  *
