@@ -18,11 +18,11 @@
  *
  * Created 11.11.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Thursday 16 May 02 1:21:17 tews@ithif56.inf.tu-dresden.de>
  *
  * substitution and optimization for top_types
  *
- * $Id: logic_util.ml,v 1.18 2002/01/24 13:39:10 tews Exp $
+ * $Id: logic_util.ml,v 1.20 2002/05/22 13:42:45 tews Exp $
  *
  *)
 
@@ -463,13 +463,6 @@ and substitute_expression ((_,eq_ex) as eqs) ((_,ex_sub) as subst) expr =
 		 if ex == nex && variants == nvariants 
 		 then expr
 		 else CCSL_Case(nex, nvariants)
-(* 	      | Reduce(adt, flag, args) ->
- * 		  let nargs = recurse_args args 
- * 		  in
- * 		    if nargs==args 
- * 		    then expr
- * 		    else Reduce(adt, flag, nargs)
- *)
 	   | Box(typ,pred,tlist) ->
 	       let n_typ =  recurse_type typ in
 	       let n_pred = recurse_exp pred in
@@ -577,13 +570,6 @@ and substitute_formula eqs subst formula =
 	   if ex_a == nex_a && ex_b == nex_b 
 	   then formula
 	   else Equal(nex_a, nex_b)
-(* 	| LessOrEqual(ex_a,ex_b) -> 
- * 	    let nex_a = recurse_exp ex_a in
- * 	    let nex_b = recurse_exp ex_b in
- * 	      if ex_a == nex_a && ex_b == nex_b 
- * 	      then formula
- * 	      else LessOrEqual(nex_a, nex_b)
- *)
      | Forall(quant_list, f) -> 
 	 let ids = List.map fst quant_list in
 	 let nsubst = bind_vars subst ids in
@@ -686,6 +672,160 @@ let substitute_arguments_types_only eq_type subst arg_list =
  * verious versions of eta 
  *)
 
+(* utility function: checks for free occurences of an identifier.
+ * takes care of bindings in Forall/Exists/Lambda. Variables bind 
+ * in cases are taken care of if they are in the form supported by
+ * extract_case_binders. 
+ * An identifier is free, if it occurs as Term(id,...) or as 
+ * BasicExpr(Termvar(... id ...))
+ *)
+
+let rec occurs_free_in_expression id expr = 
+  let recurse_exp = occurs_free_in_expression id in
+  let recurse_form = occurs_free_in_formula id 
+  in
+    (match expr with
+       | ExprLoc(ex,loc) -> recurse_exp ex
+       | BasicExpr bexp -> 
+	   (match bexp with
+	      | TermVar idc -> 
+		  (resolution_of idc).id_token.token_name = id
+	      | _ -> false
+	   )
+       | Term(name,flag,args) -> name = id
+       | TypedTerm(ex, typ) -> recurse_exp ex
+       | TypeAnnotation(ex, typ) -> recurse_exp ex
+       | QualifiedTerm(th,arg_flag,args,term) -> false
+       | MethodSelection(ex, instiface, m) -> recurse_exp ex
+       | Tuple(ex_list) -> 
+	   List.exists recurse_exp ex_list
+       | Projection(i,n) -> false
+       | RecordTuple(fields) -> 
+	   List.exists (fun (label,ex) -> recurse_exp ex) fields
+       | RecordSelection(field,ex) -> recurse_exp ex
+       | RecordUpdate(ex, changes) -> 
+	   (recurse_exp ex) or
+	   (List.exists (fun (label,ex) -> recurse_exp ex) changes)
+       | List(ex_list) -> List.exists recurse_exp ex_list
+       | Abstraction(decl_list,ex) ->
+	   if List.exists (fun (v,typ) -> v = id) decl_list
+	   then				(* id is bound *)
+	     false
+	   else 
+	     recurse_exp ex
+       | SmartAbstraction(decl_list,ex) ->
+	   if List.exists (fun (v,typ) -> v = id) decl_list
+	   then				(* id is bound *)
+	     false
+	   else 
+	     recurse_exp ex
+       | Application(ex1,ex2) -> 
+	   (recurse_exp ex1) or (recurse_exp ex2)
+       | InfixApplication(ex1,instiface,memcontainer,ex2) ->
+	   (recurse_exp ex1) or (recurse_exp ex2)
+       | SmartApplication(ex1,ex2l) ->
+	   (recurse_exp ex1) or (List.exists recurse_exp ex2l)
+       | FunUpdate(fex, changes) ->
+	   (recurse_exp fex) or
+	   (List.exists (fun (ex1, ex2) ->
+			   (recurse_exp ex1) or (recurse_exp ex2))
+	      changes)
+       | Let(decl_list, ex) ->
+	   List.fold_right
+	     (fun (lid,typopt,lex) res ->
+		if lid.id_token.token_name = id 
+		then			(* bound by this let *)
+		  recurse_exp lex
+		else			(* not bound *)
+		  (recurse_exp lex) or res
+	     )
+	     decl_list (recurse_exp ex)
+       | If(conds,ex) ->
+	   (List.exists (fun (cond,ex) -> 
+			   (recurse_form cond) or recurse_exp ex)
+	      conds)
+	   or 
+	   (recurse_exp ex)
+       | Case(ex,variants) -> 
+	   (recurse_exp ex) or
+	   (List.exists 
+	      (fun (pat,case) ->
+		 if List.exists (fun caseid -> caseid = id) 
+		   (extract_case_binders pat)
+		 then
+		   false
+		 else
+		   recurse_exp case
+	      )
+	      variants)
+       | CCSL_Case(ex,variants) -> 
+	   (recurse_exp ex) or
+	   (List.exists 
+	      (fun (memc,idreclist,case) ->
+		 if List.exists (fun idrec -> idrec.id_token.token_name = id)
+		   idreclist
+		 then
+		   false
+		 else
+		   recurse_exp case
+	      )
+	      variants)
+       | Box(typ,pred,tlist) -> recurse_exp pred
+       | Diamond(typ,pred,tlist) -> recurse_exp pred
+       | Every(typ, pred_list) -> 
+	   List.exists recurse_exp pred_list
+       | RelEvery(typ, pred_list) -> 
+	   List.exists recurse_exp pred_list
+       | Map(typ, expr_list) ->
+	   List.exists recurse_exp expr_list
+       | Expression form -> recurse_form form 
+       | Comment_str(s,ex) -> recurse_exp ex
+       | Comment_expr(ex1,typ,ex2) -> recurse_exp ex2
+       | Comprehension(s,typ,form) -> 
+	   if id = s then false
+	   else recurse_form form
+    )
+	
+and occurs_free_in_formula id formula =
+  let recurse_exp = occurs_free_in_expression id in
+  let recurse_form = occurs_free_in_formula id 
+  in
+    (match formula with
+       | FormLoc(f,loc) -> recurse_form f
+       | True -> false
+       | False -> false
+       | Not f -> recurse_form f
+       | And f_list -> List.exists recurse_form f_list
+       | Or f_list -> List.exists recurse_form f_list
+       | Implies(assum,concl) -> 
+	   (recurse_form assum) or (recurse_form concl)
+       | Iff(assum,concl) -> 
+	   (recurse_form assum) or (recurse_form concl)
+       | Equal(ex_a,ex_b) -> 
+	   (recurse_exp ex_a) or (recurse_exp ex_b)
+       | Forall(quant_list, f) -> 
+	   if List.exists (fun (v,typ) -> v = id) quant_list
+	   then false
+	   else recurse_form f
+       | Exists(quant_list, f) -> 
+	   if List.exists (fun (v,typ) -> v = id) quant_list
+	   then false
+	   else recurse_form f
+       | ConstantPredicate name -> false
+       | Formula ex -> recurse_exp ex 
+       | MetaImplies(assum,concl) -> 
+	   (recurse_form assum) or (recurse_form concl)
+       | MetaForall(quant_list, f) -> 
+	   if List.exists (fun (v,typ) -> v = id) quant_list
+	   then false
+	   else recurse_form f
+       | Bisim(typ,ex1,ex2) -> 
+	   (recurse_exp ex1) or (recurse_exp ex2)
+       | Obseq(t,ex1,ex2) -> 
+	   (recurse_exp ex1) or (recurse_exp ex2)
+    )
+  
+
 
 (* utility function: decides wether an abstraction is of a special form
  * (more precisely if it is the identity permutation), that allows to 
@@ -821,6 +961,14 @@ let rec do_opt_formula = function
 	     do_opt_formula(
 	       Forall(decllist1, 
 		      Equal(body1,body2)))
+	 | Tuple(exleft), Tuple(exright)
+	     when (List.length exleft) = (List.length exright)
+	       -> 
+	     do_opt_formula
+	       (And( List.map2 (fun exl exr -> Equal(exl, exr))
+		       exleft exright
+		   ))
+
 	 | nex1,nex2 -> 
 	     Equal( do_opt_expression nex1, 
 		    do_opt_expression nex2)
@@ -863,26 +1011,51 @@ and do_opt_expression = function
 	    Abstraction(fun_decl, 
 			Expression(Equal(f1_term, f2_term))))
   | Abstraction([],exp) -> do_opt_expression exp
-  | Abstraction([v,t] as decl, ex) ->
-      (match (do_opt_expression ex) with
-	 | Application(Term _ as ex',Term(v',Always,[]))
+(*   | Abstraction([v,t] as decl, ex) ->
+ * 	 (match (do_opt_expression ex) with
+ * 	    | Application(Term _ as ex',Term(v',Always,[]))
+ * 		when v= v' 
+ * 			-> ex'
+ * 	    | Application(Expression(ConstantPredicate _) as ex',
+ * 			  Term(v',Always,[]))
+ * 		when v= v' 
+ * 			-> ex'
+ * 	    | Term(v',Always,[]) when v = v' -> 
+ * 		Term(name_of_identity_function(), Always, [TypeArgument t])
+ * 	    | ex' -> (Abstraction(decl,ex')))      
+ *   | Abstraction(vl, ex)->
+ * 	 (match do_opt_expression ex with
+ * 	    | Application(Term _ as ex',Tuple tl)
+ * 		when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
+ * 			  -> ex'
+ * 	    | Application(Expression(ConstantPredicate _) as ex',Tuple tl)
+ * 		when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
+ * 			  -> ex'
+ * 	    | Tuple(tl) 
+ * 		when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
+ * 		  -> Term(name_of_identity_function(), Always,
+ * 			  [TypeArgument(Product(List.map snd vl))])
+ * 	    | ex' -> Abstraction(vl,ex'))
+ *)
+      
+  | Abstraction([v,t] as decl, ex)->
+      (match do_opt_expression ex with
+	 | Application(ex', Term(v',Always,[]))
 	     when v= v' 
-		     -> ex'
-	 | Application(Expression(ConstantPredicate _) as ex',
-		       Term(v',Always,[]))
-	     when v= v' 
-		     -> ex'
+	       && not (occurs_free_in_expression v ex')
+	       -> ex'
 	 | Term(v',Always,[]) when v = v' -> 
 	     Term(name_of_identity_function(), Always, [TypeArgument t])
 	 | ex' -> (Abstraction(decl,ex')))      
+
   | Abstraction(vl, ex)->
       (match do_opt_expression ex with
-	 | Application(Term _ as ex',Tuple tl)
+	 | Application(ex', Tuple tl)
 	     when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
-		       -> ex'
-	 | Application(Expression(ConstantPredicate _) as ex',Tuple tl)
-	     when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
-		       -> ex'
+	       && not (List.for_all (fun (v,t) -> 
+				       occurs_free_in_expression v ex')
+			 vl)
+	       -> ex'
 	 | Tuple(tl) 
 	     when tl = (List.map (fun (v,t) -> Term(v,Always,[])) vl)
 	       -> Term(name_of_identity_function(), Always,
@@ -907,7 +1080,7 @@ and do_opt_expression = function
 	    | Adt(_,_,_) -> do_opt_expression(Expression(Equal(tl,tr)))
 	    | Groundtype _ -> do_opt_expression(Expression(Equal(tl,tr)))
 		  (* other things must not occur under RelEvery *)
-	    | _ -> assert(false); raise Internal_error
+	    | _ -> assert(false)
 	else
 	  Application(
 	    RelEvery(typ, forml'),
@@ -950,7 +1123,28 @@ and do_opt_expression = function
 	 | x -> Expression x)
   | Projection(i,n) -> Projection(i,n)
   | Tuple exl -> 
-      Tuple (List.map do_opt_expression exl)
+      (match List.map do_opt_expression exl with
+	 | [] -> Tuple []
+	 | (Application(Projection(1, n), tt)) :: oexl' as exl' ->
+	     if (List.length exl') = n
+	       && snd (List.fold_left 
+			 (fun (i,res) -> function 
+			    | Application(Projection(j,m) , tt')
+				when j = i && m = n && tt == tt'
+				  -> (i+1,res)
+			    | _ -> (i+1, false)
+			 )
+			 (2, true) oexl'
+		      )
+	     then			(* (Proj_1 t, Proj_2 t, Proj_3 t) *)
+	       tt
+	     else
+	       Tuple exl'
+	 | exl' -> Tuple exl'
+      )
+(* 
+ *   | Tuple exl -> Tuple (List.map do_opt_expression exl)
+ *)
   | RecordTuple lexl -> 
       RecordTuple (List.map
 		     (fun (l,ex) -> (l, do_opt_expression ex))
@@ -1002,6 +1196,7 @@ let opt_expression f =
   else f
 
 
+
 (***********************************************************************
  * 
  * manipulate variances
@@ -1046,7 +1241,7 @@ let rec variance_subst v1 v2 = match v1,v2 with
   | Neg,Pos -> Neg
   | Neg,Neg -> Pos
   | Unset,_
-  | _, Unset -> assert(false); raise Internal_error
+  | _, Unset -> assert(false)
 
 
 let rec variance_join v1 v2 = match v1,v2 with
@@ -1063,7 +1258,7 @@ let rec variance_join v1 v2 = match v1,v2 with
   | Neg,Pos -> Mixed
   | Neg,Neg -> Neg
   | Unset,_
-  | _, Unset -> assert(false); raise Internal_error
+  | _, Unset -> assert(false)
 
 
 let variance_flatten vlist = 
@@ -1072,7 +1267,7 @@ let variance_flatten vlist =
 	  | None, Some a -> a :: accu
 	  | Some a, None -> a :: accu
 	  | Some a, Some a' -> a :: a' :: accu
-	  | _ -> (assert(false); raise Internal_error)
+	  | _ -> assert(false)
     )
     vlist []
 
@@ -1091,25 +1286,15 @@ let var_terms_from_var_ids =
 	   | None, Some(name, _) -> (id, (None, build_term name))
 	   | Some (name1,_), Some(name2,_) ->
 	       (id, (build_term name1, build_term name2))
-	   | _ -> (assert(false); raise Internal_error)
+	   | _ -> assert(false)
 	)
 	type_param_ids var_ids
 
-(* HH
- *     (* flatten a list of identifiers with variance information 
- * 	* and convert them into an expression list, ready 
- * 	* for application
- * 	*)
- * let arg_terms_from_var_ids var_ids = 
- *   List.map 
- *     (fun (id,t) -> Term(id,Always,[]))
- *     (variance_flatten var_ids)
- * 
- *)
 
 (*** Local Variables: ***)
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
